#    Copyright 2015 Mirantis, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import click


class AliasedGroup(click.Group):
    """This class introduces iproute2-like behaviour,

    command will be inferredby matching patterns.
    If there will be more than 1 matches - exception will be raised

    Examples:
    >> solar ch stage
    >> solar cha process
    >> solar res action run rabbitmq_service1
    """

    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))


class EGroup(click.Group):

    error_wrapper_enabled = False

    error_wrapper = None

    def add_command(self, cmd, name=None):
        if self.error_wrapper:
            cmd.callback = self.error_wrapper(cmd.callback)
        return super(EGroup, self).add_command(cmd, name)
